import 'package:adjeminpay_flutter/adjeminpay_flutter.dart';
import 'package:flutter/material.dart';
import 'package:uuid/uuid.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatelessWidget {

  const MyApp({Key? key}) : super(key: key);

  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        // This is the theme of your application.
        //
        // Try running your application with "flutter run". You'll see the
        // application has a blue toolbar. Then, without quitting the app, try
        // changing the primarySwatch below to Colors.green and then invoke
        // "hot reload" (press "r" in the console where you ran "flutter run",
        // or simply save your changes to "hot reload" in a Flutter IDE).
        // Notice that the counter didn't reset back to zero; the application
        // is not restarted.
        primarySwatch: Colors.blue,
      ),
      home: const MyHomePage(title: 'Flutter Demo Home Page'),
    );
  }
}

class MyHomePage extends StatefulWidget {
  const MyHomePage({Key? key, required this.title}) : super(key: key);

  // This widget is the home page of your application. It is stateful, meaning
  // that it has a State object (defined below) that contains fields that affect
  // how it looks.

  // This class is the configuration for the state. It holds the values (in this
  // case the title) provided by the parent (in this case the App widget) and
  // used by the build method of the State. Fields in a Widget subclass are
  // always marked "final".

  final String title;

  @override
  State<MyHomePage> createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {

  final uuid = Uuid();

  @override
  Widget build(BuildContext context) {
    // This method is rerun every time setState is called, for instance as done
    // by the _incrementCounter method above.
    //
    // The Flutter framework has been optimized to make rerunning build methods
    // fast, so that you can just rebuild anything that needs updating rather
    // than having to individually change instances of widgets.
    return Scaffold(
      appBar: AppBar(
        // Here we take the value from the MyHomePage object that was created by
        // the App.build method, and use it to set our appbar title.
        title: Text(widget.title),
      ),
      body: Center(
        // Center is a layout widget. It takes a single child and positions it
        // in the middle of the parent.
        child: Column(
          // Column is also a layout widget. It takes a list of children and
          // arranges them vertically. By default, it sizes itself to fit its
          // children horizontally, and tries to be as tall as its parent.
          //
          // Invoke "debug painting" (press "p" in the console, choose the
          // "Toggle Debug Paint" action from the Flutter Inspector in Android
          // Studio, or the "Toggle Debug Paint" command in Visual Studio Code)
          // to see the wireframe for each widget.
          //
          // Column has various properties to control how it sizes itself and
          // how it positions its children. Here we use mainAxisAlignment to
          // center the children vertically; the main axis here is the vertical
          // axis because Columns are vertical (the cross axis would be
          // horizontal).
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            const Text(
              'You have pushed the button this many times:',
            ),
          ],
        ),
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: (){

          runTransaction();
        },
        tooltip: 'Increment',
        child: const Icon(Icons.add),
      ), // This trailing comma makes auto-formatting nicer for build methods.
    );
  }

  void runTransaction()async {

    final Map<String, dynamic> paymentResult = await Navigator.push(context, MaterialPageRoute(builder: (context)=>AdjeminPay(
      // ! required clientId
      clientId: Constants.ADJEMINPAY_CLIENT_ID,
      // ! required clientScret
      clientSecret: Constants.ADJEMINPAY_CLIENT_SECRET,
      // ! required transactionId required
      // for you to follow the transaction
      //    or retrieve it later
      //    should be a string < 191 and unique for your application
      merchantTransactionId: uuid.v4(),
      // ! required designation
      //   the name the user will see as what they're paying for
      designation: 'Payer une commande',
      // notifyUrl for your web backend
      notificationUrl:Constants.ADJEMINPAY_NOTIFICATION_URL,
      // amount: int.parse("${orderData['totalAmount']}"),
      // ! required amount
      //    amount the user is going to pay
      //    should be an int
      amount: 100,
      // currency code
      // currently supported currency is XOF
      currencyCode: 'XOF',
      // designation: widget.element.title,
      // the name of your user
      buyerName: "Ange Bagui", //optional
    )));

    print(">>> ADJEMINPAY PAYMENT RESULTS <<<");
    print(paymentResult);
    // * Here you define your callbacks
    // Callback if the paymentResult is null
    //    the payment gate got closed without sending back any data
    if (paymentResult == null) {
      print("<<< Payment Gate Unexpectedly closed");
      return;
    }
    Scaffold.of(context).showSnackBar(SnackBar(content: Text("Payment Status is ${paymentResult['status']}")));
    // Callback on payment successfully
    if (paymentResult['status'] == "SUCCESSFUL") {
      print("<<< AdjeminPay success");
      print(paymentResult);
      // redirect to or show another screen
      return;
    }
    // Callback on payment failed
    if (paymentResult['status'] == "FAILED") {
      print("<<< AdjeminPay failed");
      print(paymentResult);
      // the reason with be mentionned in the paymentResult['message']
      print("Reason is : " + paymentResult['message']);
      // redirect to or show another screen
      return;
    }
    // Callback on payment cancelled
    if (paymentResult['status'] == "CANCELLED") {
      print("<<< AdjeminPay cancelled");
      print(paymentResult);
      // the reason with be mentionned in the paymentResult['message']
      print("Reason is : " + paymentResult['message']);
      // redirect to or show another screen
      return;
    }
    // Callback on payment cancelled
    if (paymentResult['status'] == "EXPIRED") {
      print("<<< AdjeminPay expired");
      print(paymentResult);
      // The user took too long to approve or refuse payment
      print("Reason is : " + paymentResult['message']);
      // redirect to or show another screen
      return;
    }
    // Callback on initialisation error
    if (paymentResult['status'] == "INVALID_PARAMS") {
      print("<<< AdjeminPay Init error");
      // You didn't specify a required field
      print(paymentResult);
      return;
    }

    if (paymentResult['status'] == "INVALID_CREDENTIALS") {
      print("<<< AdjeminPay Init error");
      // You didn't specify a required field
      // or your clientId or clientSecret are not valid
      print(paymentResult);
      return;
    }
    // Callback when AdjeminPay requests aren't completed
    if (paymentResult['status'] == "ERROR_HTTP") {
      return;
    }

  }
}

class Constants{

  static final String ADJEMINPAY_CLIENT_ID = '154';//"41";
  static final String ADJEMINPAY_CLIENT_SECRET = "3SwqRnZ8pTUdYKqWmqjFrnY5Gdu8mGrHqRq2Gd4C";//"Y4R91969G3GYKV1JKvKQaaliK95yluEWKbHKPrfj";
  static final String ADJEMINPAY_NOTIFICATION_URL = "https://your.backend.api/notifyUrl";
}
